@@ -27,9 +27,6 @@ int __lyricwiki_get_soap_message(post_message *soap, gchar *artist, gchar *songt
 	if(!artist || !songtitle)
 		return 0;
 	
-	/* escape this */
-	songtitle = g_markup_escape_text(songtitle, -1); 
-	artist = g_markup_escape_text(artist, -1);
 	/* create request */
 	GString *msg;
 	msg = g_string_new("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
@@ -46,10 +43,7 @@ int __lyricwiki_get_soap_message(post_message *soap, gchar *artist, gchar *songt
 	g_string_append(msg, artist); 
 	g_string_append(msg, "</artist><song xsi:type=\"xsd:string\">");
 	g_string_append(msg, songtitle);
-	g_string_append(msg, "</song></tns:getSong></SOAP-ENV:Body></SOAP-ENV:Envelope>\n");
-
-	g_free(artist); 
-	g_free(songtitle); 
+	g_string_append(msg, "</song></tns:getSong></SOAP-ENV:Body></SOAP-ENV:Envelope>\n"); 
 	
 	soap->body = msg; 
 	add_post_header(soap, "SOAPAction: urn:LyricWiki#getSong");
