@@ -19,6 +19,7 @@
  */
 
 #include "gmpc-lyrics.h"
+#include <config.h>
 
 #define LYRICS_FROM "\n\nLyric from "
 
@@ -27,6 +28,9 @@ static GtkWidget *lyrics_pref_table = NULL;
 
 static xmlGenericErrorFunc handler = NULL;
 
+static gchar *escape_uri_string (const gchar *string);
+static int shrink_string(gchar *string, int start, int end);
+
 int lyrics_get_enabled();
 void lyrics_set_enabled(int enabled);
 /* Playlist window row reference */
@@ -55,47 +59,35 @@ static int fetch_priority()
  * Preferences structure
  */
 gmpcPrefPlugin lyrics_gpp = {
-	lyrics_construct,
-	lyrics_destroy
+	.construct  = lyrics_construct,
+	.destroy    = lyrics_destroy
 };
 
 /**
  * Meta data structure 
  */
-
 gmpcMetaDataPlugin lyric_fetch = {
-	fetch_priority,
-	fetch_lyric
+	.get_priority   = fetch_priority,
+	.get_image      = fetch_lyric
 };
+
 gmpcPlugin plugin = {
 	/** Name */
-	"Lyrics fetcher",
+	.name           = "Lyrics fetcher",
 	/** Version */
-	{0,15,0},
+	.version        = {PLUGIN_MAJOR_VERSION,PLUGIN_MINOR_VERSION,PLUGIN_MICRO_VERSION},
 	/** Plugin type */
-	GMPC_PLUGIN_META_DATA,
-	/** internal id */
-	0,
-	/** path where the plugin is located (filled in by gmpc) */
-	NULL, 
+	.plugin_type    = GMPC_PLUGIN_META_DATA,
 	/** initialize function */
-	&lyrics_init, 
-        /* Destroy */
-        NULL,
-	/** Browser plugin */
-	NULL,
-	/** Status Changed signals */
-	NULL, 
-	/** Connection changed */
-	NULL, 
+	.init           = &lyrics_init, 
 	/** Preferences structure */
-	&lyrics_gpp,
+	.pref           = &lyrics_gpp,
 	/** Meta data structure */
-	&lyric_fetch,
+	.metadata       = &lyric_fetch,
 	/** get_enabled()  */
-	lyrics_get_enabled,
+	.get_enabled    = lyrics_get_enabled,
 	/** set enabled() */
-	lyrics_set_enabled
+	.set_enabled    = lyrics_set_enabled
 };
 
 static struct lyrics_api apis[] =
@@ -174,7 +166,7 @@ void free_post_message(post_message *s) {
 
 void lyrics_init (void)
 {
-  gchar *path = g_strdup_printf("%s/.lyrics/", g_get_home_dir());
+/*  gchar *path = g_build_path(G_DIR_SEPARATOR_S, g_get_home_dir(),".lyrics",NULL);
 
   if (!g_file_test(path, G_FILE_TEST_IS_DIR))
   {
@@ -182,6 +174,7 @@ void lyrics_init (void)
   }
 
   g_free(path);
+  */
 }
 
 xmlNodePtr get_node_by_name (xmlNodePtr node, xmlChar *name)
@@ -196,52 +189,9 @@ xmlNodePtr get_node_by_name (xmlNodePtr node, xmlChar *name)
   return NULL;
 }
 
-/* i think this interestingly named function urlencodes string */
 static char * __lyrics_process_string(char *name)
 {
-	int i = 0;
-	int j = 0;
-	int depth = 0;
-	char *result = NULL;
-
-	int length = strlen(name);
-	for(i=0; i<strlen(name);i++)
-	{
-		if(!(name[i] >= 'a' && name[i] <= 'z') &&
-				!(name[i] >= 'A' && name[i] <= 'Z') &&
-				!(name[i] >= '0' && name[i] <= '9'))
-		{
-			length+=2;
-		}
-
-	}
-	/** allocate */
-	/** TODO do this somewhere else */
-	result = g_malloc0((length+1)*sizeof(char));
-	depth = 0;
-	for(i=0; i < strlen(name);i++) {
-		if(name[i] == '(' || name[i] == '[' || name[i] == '{')
-		{
-			depth++;
-		}
-		else if(name[i] == ')' || name[i] == ']' || name[i] == '}') {
-			depth--;
-		}
-		else if(depth >0)
-		{
-			/* do nothing */
-		}
-		else if(!(name[i] >= 'a' && name[i] <= 'z') &&
-				!(name[i] >= 'A' && name[i] <= 'Z') &&
-				!(name[i] >= '0' && name[i] <= '9')) {
-			snprintf(&result[j],4,"%%%02X", name[i]);
-			j+=3;
-		} else {
-			result[j] = name[i];
-			j++;
-		}
-	}
-	return result;
+	return escape_uri_string(name);
 }
 
 static int fetch_lyrics (mpd_Song *song, struct lyrics_api *api, gchar **lyrics, int exact)
@@ -283,6 +233,7 @@ static int fetch_lyrics (mpd_Song *song, struct lyrics_api *api, gchar **lyrics,
 	  g_free(esc_title);
   }
   debug_printf(DEBUG_INFO, "search url:'%s'\n", search_url);
+
   if (!gmpc_easy_download (search_url, &dl))
   {
 	  g_free(search_url);
@@ -427,8 +378,6 @@ int do_post (post_message *msg) {
 	c_ret = curl_easy_perform(con); 
 
 	curl_easy_getinfo(con, CURLINFO_RESPONSE_CODE, &(msg->response_code)); 
-
-	//printf(msg->response->str);
 	
 	/* do curl cleanup now */
 	curl_slist_free_all(msg->headers);
@@ -455,8 +404,12 @@ static int fetch_lyrics_soap(mpd_Song *song, struct lyrics_api *api, gchar **lyr
       		return 1;
   	}
 	
-	init_post_message(&request); 
-	ret = api->get_soap_message(&request, song->artist, song->title); 
+	init_post_message(&request);
+	char *esc_artist = __lyrics_process_string(song->artist);
+	char *esc_title = __lyrics_process_string(song->title);
+	ret = api->get_soap_message(&request, esc_artist, esc_title);
+	g_free(esc_artist);
+	g_free(esc_title); 
 	
 	if(!ret) {
 		free_post_message(&request); 
@@ -552,8 +505,8 @@ int fetch_lyric(mpd_Song *song, MetaDataType type, char **path)
 		ret = fetch_lyric_loop(song, &lyrics, id, exact);
 		if(!ret  && lyrics && strlen(lyrics))
 		{
-			*path = g_strdup_printf("%s/.lyrics/%s-%s.txt", g_get_home_dir(),
-					song->artist, song->title);
+			gchar *filename = g_strdup_printf("%s-%s.lyric", song->artist, song->title);
+			*path = gmpc_get_covers_path(filename); 
 			g_file_set_contents(*path, lyrics, -1, NULL);                      				
 			g_free(lyrics);
 			return META_DATA_AVAILABLE;
@@ -657,3 +610,106 @@ void lyrics_set_enabled(int enabled)
 	cfg_set_single_value_as_int(config, "lyrics-plugin", "enable", enabled);
 }
 
+static gchar *escape_uri_string (const gchar *string)
+{
+#define ACCEPTABLE(a) (((a) >= 'a' && (a) <= 'z') || ((a) >= 'A' && (a) <= 'Z') || ((a) >= '0' && (a) <= '9'))
+
+	const gchar hex[16] = "0123456789ABCDEF";
+	const gchar *p;
+	gchar *q;
+	gchar *result;
+	int c;
+	gint unacceptable = 0;
+	const gchar *tmp_p;
+	gchar *new_string;
+	int depth = 0;
+	int len;
+	int i = 0;
+
+	len = strlen(string);
+
+	new_string = g_malloc(len + 1);
+
+	/*	Get count of chars that will need to be converted to %
+		and remove ([{}]) and everything between */
+	for (p = string; *p != '\0'; p++) {
+		c = (guchar) *p;
+
+		if(c == '(' || c == '[' || c == '{') {
+			depth++;
+		} else if(c == ')' || c == ']' || c == '}') {
+			depth--;
+			if(depth < 0)
+				depth = 0;
+		} else if(depth == 0) {
+			if (!ACCEPTABLE (c)) 
+				unacceptable++;
+
+			new_string[i] = c;
+
+			i++;
+		}
+	}
+
+	new_string[i] = '\0';
+
+	len = strlen(new_string);
+
+	/* remove double spaces from the string because removing ([{}])
+		tends to create those */
+	for(p = new_string + 1; *p != '\0'; p++) {
+		c = (guchar) *p;
+		if(c == ' ') {
+			tmp_p = p - 1;
+			if( *tmp_p == ' ') {
+				len = shrink_string(new_string,  p - new_string, len);
+				p--;
+			}
+		}
+	}
+	
+	/* make sure first char isn't a space */
+	if(new_string[0] == ' ')
+		len = shrink_string(new_string, 0, len);
+
+	/* make sure there isn't a trailing space*/
+	if(new_string[len - 1] == ' ')
+	len--;
+
+	new_string[len] = '\0';
+
+	result = g_malloc (len + unacceptable * 2 + 1);
+
+	/*time to create the escaped string*/
+	for (q = result, p = new_string; *p != '\0'; p++)
+	{
+		c = (guchar) *p;
+
+		if (!ACCEPTABLE (c)) {
+			*q++ = '%'; /* means hex coming */
+			*q++ = hex[c >> 4];
+			*q++ = hex[c & 15];
+		}
+		else
+			*q++ = *p;
+	}
+
+	*q = '\0';
+
+	g_free(new_string);
+
+	return result;
+}
+
+static int shrink_string(gchar *string, int start, int end)
+{
+	int i;
+	
+	for( i = start; i < end; i++)
+		string[i] = string[i + 1];
+		
+	end--;
+	
+	return end;
+}
+
